@@ -19,6 +19,9 @@ logging.debug('Adding Reference Microsoft.AnalysisServices')
 clr.AddReference('Microsoft.AnalysisServices')
 
 logging.debug(f"Importing from the rest...")
-from . pytabular import Tabular, BPA, TE2
+from . pytabular import Tabular
 from . basic_checks import Return_Zero_Row_Tables, Table_Last_Refresh_Times, BPA_Violations_To_DF
-from . logic_utils import pd_dataframe_to_m_expression, pandas_datatype_to_tabular_datatype
\ No newline at end of file
+from . logic_utils import pd_dataframe_to_m_expression, pandas_datatype_to_tabular_datatype
+from . tabular_tracing import Base_Trace, Refresh_Trace
+from . tabular_editor import Tabular_Editor
+from . best_practice_analyzer import BPA
\ No newline at end of file
