@@ -1,11 +1,10 @@
 import logging
-logging.basicConfig(level=logging.DEBUG,format='%(asctime)s :: %(module)s :: %(levelname)s :: %(message)s')
-
 import datetime
+import os
 from typing import Dict, List
 import pandas as pd
-
 import clr
+
 clr.AddReference('Microsoft.AnalysisServices.Tabular')
 from Microsoft.AnalysisServices.Tabular import DataType
 
@@ -75,25 +74,30 @@ def pd_dataframe_to_dax_expression(df:pd.DataFrame = pd.DataFrame(data={'col1':
 		return
 	return True
 def pd_dataframe_to_m_expression(df:pd.DataFrame) -> str:
-	'''
-	This will take a pandas dataframe and convert to an m expression
-	For example this DF:
-	   col1  col2
-	0   1     3
-	1   2     4
-	
-	|
-	|
-	V
-
-	Will convert to this expression string:
-	let
-	Source=#table({"col1","col2"},
-	{
-	{"1","3"},{"2","4"}
-	})
-	in
+	'''This will take a pandas dataframe and convert to an m expression  
+	For example this DF:  
+	   col1  col2  
+	0   1     3  
+	1   2     4  
+	  
+	|  
+	|  
+	V  
+  
+	Will convert to this expression string:  
+	let  
+	Source=#table({"col1","col2"},  
+	{  
+	{"1","3"},{"2","4"}  
+	})  
+	in  
 	Source
+
+	Args:
+		df (pd.DataFrame): Pandas DataFrame
+
+	Returns:
+		str: Currently only returning string values in your tabular model.
 	'''
 	def m_list_expression_generator(list_of_strings:List[str]) -> str:
 		'''
@@ -111,4 +115,15 @@ def pd_dataframe_to_m_expression(df:pd.DataFrame) -> str:
 	for index, row in df.iterrows():
 		expression_list_rows += [m_list_expression_generator(row.to_list())]
 	expression_str += f"\u007b\n{','.join(expression_list_rows)}\n\u007d)\nin\nSource"
-	return expression_str
\ No newline at end of file
+	return expression_str
+
+def remove_folder_and_contents(folder_location):
+	'''Internal used in tabular_editor.py and best_practice_analyzer.py.
+
+	Args:
+		folder_location (str): Folder path to remove directory and contents.
+	'''
+	import shutil
+	if os.path.exists(folder_location):
+		logging.info(f'Removing Dir and Contents -> {folder_location}')
+		shutil.rmtree(folder_location)
\ No newline at end of file
