@@ -1,14 +1,14 @@
 import logging
 
 logging.debug(f'Importing Microsoft.AnalysisServices.Tabular')
-from Microsoft.AnalysisServices.Tabular import Server, Database, RefreshType, DataType, ConnectionDetails, ColumnType, MetadataPermission, Table, DataColumn, Partition, MPartitionSource, PartitionSourceType
+from Microsoft.AnalysisServices.Tabular import Server, Database, RefreshType, DataType, ConnectionDetails, ColumnType, MetadataPermission, Table, DataColumn, Partition, MPartitionSource, PartitionSourceType, Trace, TraceEvent, TraceEventHandler
 logging.debug(f'Importing Microsoft.AnalysisServices.AdomdClient')
 from Microsoft.AnalysisServices.AdomdClient import (AdomdCommand, AdomdConnection)
 logging.debug(f'Importing Microsoft.AnalysisServices')
-from Microsoft.AnalysisServices import UpdateOptions
+from Microsoft.AnalysisServices import UpdateOptions, TraceEventClass, TraceEventSubclass, TraceEventCollection, TraceColumn
 
 logging.debug('Importing Other Packages...')
-from typing import List, Union
+from typing import List, Union, Callable
 from collections.abc import Iterable
 import requests as r
 import pandas as pd
@@ -17,9 +17,10 @@ import os
 import subprocess
 import atexit
 from logic_utils import pd_dataframe_to_m_expression, pandas_datatype_to_tabular_datatype
+from tabular_tracing import Refresh_Trace
 
 class Tabular:
-	'''Tabular Class to perform operations:[Microsoft.AnalysisServices.Tabular](https://docs.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.tabular?view=analysisservices-dotnet)
+	'''Tabular Class to perform operations: [Microsoft.AnalysisServices.Tabular](https://docs.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.tabular?view=analysisservices-dotnet)
 
 	Args:
 		CONNECTION_STR (str): [Connection String](https://docs.microsoft.com/en-us/analysis-services/instances/connection-string-properties-analysis-services?view=asallproducts-allversions)
@@ -76,7 +77,7 @@ class Tabular:
 		else:
 			logging.debug(f'Disconnect Successful')
 			return True	
-	def Refresh(self, Object:Union[str,Table,Partition,Iterable], RefreshType=RefreshType.Full) -> None:
+	def Refresh(self, Object:Union[str,Table,Partition,Iterable], RefreshType=RefreshType.Full, Run:bool = True) -> None:
 		'''Input Object(s) to be refreshed in the tabular model. Combine with .SaveChanges() to actually run the refresh on the model.
 
 		Args:
@@ -84,6 +85,7 @@ class Tabular:
 			RefreshType (_type_, optional): [RefreshType](https://docs.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.tabular.refreshtype?view=analysisservices-dotnet). Defaults to RefreshType.Full.
 		'''
 		logging.debug(f'Beginning RequestRefresh cadence...')
+
 		def refresh(object):
 			if isinstance(object,str):
 				logging.info(f'Requesting refresh for {object}')
@@ -92,11 +94,19 @@ class Tabular:
 			else:
 				logging.info(f'Requesting refresh for {object.Name}')
 				object.RequestRefresh(RefreshType)
-		if isinstance(Object,Iterable):
+		
+		
+		if isinstance(Object,Iterable) and isinstance(Object,str) == False:
 			[refresh(object) for object in Object]
 		else:
 			refresh(Object)
-			Object.RequestRefresh(RefreshType)
+      
+		if Run:
+			rt = Refresh_Trace(self)
+			rt.Start()
+			self.SaveChanges()
+			rt.Stop()
+			rt.Drop()
 	def Update(self, UpdateOptions:UpdateOptions =UpdateOptions.ExpandFull) -> None:
 		'''[Update Model](https://docs.microsoft.com/en-us/dotnet/api/microsoft.analysisservices.majorobject.update?view=analysisservices-dotnet#microsoft-analysisservices-majorobject-update(microsoft-analysisservices-updateoptions))
 
@@ -199,9 +209,6 @@ class Tabular:
 
 		Returns:
 			bool: Returns True if Successful, else will return error.
-		'''		
-		'''
-
 		'''
 		logging.info(f'Beginning Revert for {table_str}')
 		logging.debug(f'Finding original {table_str}')
@@ -327,8 +334,8 @@ class Tabular:
 		return self.Query(query_str)
 	def Analyze_BPA(self,Tabular_Editor_Exe:str,Best_Practice_Analyzer:str) -> List[str]:
 		'''Takes your Tabular Model and performs TE2s BPA. Runs through Command line.
-		https://docs.tabulareditor.com/te2/Best-Practice-Analyzer.html
-		https://docs.tabulareditor.com/te2/Command-line-Options.html
+		[Tabular Editor BPA](https://docs.tabulareditor.com/te2/Best-Practice-Analyzer.html)
+		[Tabular Editor Command Line Options](https://docs.tabulareditor.com/te2/Command-line-Options.html)
 
 		Args:
 			Tabular_Editor_Exe (str): TE2 Exe File path. Feel free to use class TE2().EXE_Path or provide your own.
@@ -387,63 +394,3 @@ class Tabular:
 		self.Refresh([new_table])
 		self.SaveChanges()
 		return True
-
-
-
-class BPA:
-	'''_summary_
-	'''	
-	'''Best Practice Analyzer Class. Can provide Url, Json File Path, or Python List. If nothing is provided it will default to Microsofts Analysis Services report with BPA Rules. 
-	[Default BPA](https://raw.githubusercontent.com/microsoft/Analysis-Services/master/BestPracticeRules/BPARules.json)
-	'''
-	def __init__(self,rules_location:str='https://raw.githubusercontent.com/microsoft/Analysis-Services/master/BestPracticeRules/BPARules.json') -> None:
-		'''
-		'''
-		self.Location = rules_location
-		self.Rules:list[dict()] = []
-		logging.debug(f'Initializing BPA Class with: {rules_location}')
-		try:
-			logging.debug(f'Searching for Rules on the good ol\' internet...')
-			self.Rules = r.get(rules_location).json()
-			logging.debug(f'Rules recieved from: {rules_location}')
-		except:
-			logging.debug(f'Request to {rules_location} failed...')
-			logging.debug(f'Searching for Rules locally with file path...')
-			with open(rules_location,'r') as json_file:
-				self.Rules = json.load(json_file)
-			logging.debug(f'Rules from file path collected...')
-		pass
-#Todo... subclass with a namedtuple
-class TE2:
-	'''TE2 Class, to use any built TabularEditor Command Line Scripts  
-	[TE2 Command Line Example](https://docs.tabulareditor.com/te2/Command-line-Options.html)  
-	[TE2 Download](https://github.com/TabularEditor/TabularEditor/releases/download/2.16.7/TabularEditor.Portable.zip)
-	'''
-	def __init__(self,TE_Location='https://github.com/TabularEditor/TabularEditor/releases/download/2.16.7/TabularEditor.Portable.zip') -> None:
-		logging.debug(f'Checking for TE2 in {os.getcwd()}')
-		te2_path = os.path.join(os.getcwd(),'TE2')
-		if os.path.exists(te2_path) == False:
-			logging.debug('Downloading Tabular Editor for BPA...')
-			self.TE_Location = TE_Location
-			response = r.get(self.TE_Location)
-			file_location = f"{os.getcwd()}\\{self.TE_Location.split('/')[-1]}"
-			with open(file_location,'wb') as te2:
-				te2.write(response.content)
-			logging.debug('TE2 Zip Download Complete!')
-			logging.debug('Import ZipFile')
-			import zipfile as Z
-			logging.debug('Unzipping file...')
-			with Z.ZipFile(file_location) as zip:
-				zip.extractall(path=te2_path)
-			logging.debug('All Unzipped!')
-			logging.debug('Removing Zip File')
-			os.remove(file_location)
-		else:
-			logging.debug(f'TE2 Directory Found with Files {os.listdir(path=te2_path)}')
-		self.EXE_Path = os.path.join(te2_path,'TabularEditor.exe')
-		if os.path.exists(self.EXE_Path):
-			logging.debug(f'TabularEditor.exe Located! {self.EXE_Path}')
-		else:
-			logging.error('TabularEditor.exe not found!')
-		pass
-	pass
